@@ -24,8 +24,11 @@
 static int using_SPI;
 
 static inline TaggedTypeClass
-DetermineTaggedTypeClass( Size typlen, bool typbyval )
+DetermineTaggedTypeClass( struct TaggedTypeInfo *tti )
 {
+	Size typlen = tti->typlen;
+	bool typbyval = tti->typbyval;
+	
 	if( typbyval )
 		return 0;
 	if( typlen > 0 )
@@ -80,31 +83,6 @@ GetDLLPath()
 	return dll_path = SPI_getvalue( SPI_tuptable->vals[0], SPI_tuptable->tupdesc, 1 );
 }
 
-static inline Oid
-procLookupRettype( Oid procoid )
-{
-	return get_func_rettype( procoid );
-}
-
-static Oid
-procLookupArgType( Oid procoid )
-{
-// get_func_signature was introduced in lsyscache.h v1.76 (2003/07/01)
-// prototype changed in v1.95 (2005/03/29)
-#if CATALOG_VERSION_NO < 200503290
-	Oid	args[FUNC_MAX_ARGS];
-	int	nargs;
-	
-	get_func_signature( procoid, args, &nargs );
-#else
-	Oid	*args;
-	int	nargs;
-	
-	get_func_signature( procoid, &args, &nargs );
-#endif
-	return args[0];
-}
-
 #ifdef DEBUG
 static void
 debugDump( void *ptr, int len )
@@ -174,7 +152,7 @@ find_tag( char *data, char **str, char **tag )
  	return;
 }
 
-static Datum
+static inline Datum
 ComposeTaggedTypeDatum( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 {
 	struct taggedtyped *td = tt_palloc( sizeof( struct taggedtyped ) );
@@ -185,7 +163,7 @@ ComposeTaggedTypeDatum( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 	return PointerGetDatum( td );
 }
 
-static Datum
+static inline Datum
 ComposeTaggedTypeFixed( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 {
 	struct taggedtypef *tf = tt_palloc( tt_class + sizeof(Oid) );
@@ -196,7 +174,7 @@ ComposeTaggedTypeFixed( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 	return PointerGetDatum( tf );
 }
 
-static Datum
+static inline Datum
 ComposeTaggedTypeVar( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 {
 	struct taggedtypev *tv = tt_palloc( VARSIZE( datum ) + sizeof(Oid) );
@@ -210,9 +188,11 @@ ComposeTaggedTypeVar( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 
 /* The tt_class is the class of the passed datum (the base type). 
  * This function takes the base type and creates the tagged type. */
-static inline Datum
-ComposeTaggedType( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
+Datum
+ComposeTaggedType( struct TaggedTypeInfo *tti, Datum datum, Oid tagoid )
 {
+	TaggedTypeClass tt_class = DetermineTaggedTypeClass( tti );
+
 //	elog( NOTICE, "ComposeTaggedType( %d, %p, %d )", tt_class, DatumGetPointer(datum), tagoid );
 	if( isTaggedTypeDatum( tt_class ) )
   		return ComposeTaggedTypeDatum( tt_class, datum, tagoid );
@@ -226,9 +206,11 @@ ComposeTaggedType( TaggedTypeClass tt_class, Datum datum, Oid tagoid )
 }
 
 /* The tt_class is the class of the base type, although you pass the tagged type */
-static inline Oid
-ExtractTaggedTypeOid( TaggedTypeClass tt_class, Datum datum )
+Oid
+ExtractTaggedTypeOid( struct TaggedTypeInfo *tti, Datum datum )
 {
+	TaggedTypeClass tt_class = DetermineTaggedTypeClass( tti );
+
 	if( isTaggedTypeDatum( tt_class ) )
   		return ((struct taggedtyped*)DatumGetPointer( datum ))->tag;
   	if( isTaggedTypeFixed( tt_class ) )
@@ -242,9 +224,11 @@ ExtractTaggedTypeOid( TaggedTypeClass tt_class, Datum datum )
 }
 
 /* The tt_class is the class of the base type, although you pass the tagged type */
-static inline Datum
-ExtractTaggedTypeDatum( TaggedTypeClass tt_class, Datum datum )
+Datum
+ExtractTaggedTypeDatum( struct TaggedTypeInfo *tti, Datum datum )
 {
+	TaggedTypeClass tt_class = DetermineTaggedTypeClass( tti );
+
 	if( isTaggedTypeDatum( tt_class ) )
   		return ((struct taggedtyped*)DatumGetPointer( datum ))->val;
   	if( isTaggedTypeFixed( tt_class ) )
@@ -301,11 +285,11 @@ _taggedtype_in( char *data, Oid returnoid )
 	tag_text = DatumGetTextP( DirectFunctionCall1( textin, CStringGetDatum( tag ) ) );
 
 	datumSize = datumGetSize( datum, tti->typbyval, tti->typlen );
-	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
+	tt_class = DetermineTaggedTypeClass( tti );
 
-	parsetag = FindTagByName( tti->tagtable, tag_text );
+	parsetag = FindTagByName( tti, tag_text );
 
-	return_datum = ComposeTaggedType( tt_class, datum, parsetag );
+	return_datum = ComposeTaggedType( tti, datum, parsetag );
 
 	PG_RETURN_DATUM( return_datum );
 }
@@ -361,7 +345,7 @@ Datum taggedtype_out(PG_FUNCTION_ARGS)
 	 * function altogether with makes the point moot... */
 	 
 	if( argoid == 0 )
-		argoid = procLookupArgType( fcinfo->flinfo->fn_oid );
+		argoid = procLookupArgType( fcinfo->flinfo->fn_oid, 0 );
 	if( argoid == 0 )
 		ereport( ERROR, (errmsg("INTERNAL: Couldn't determine argument type for function %d", fcinfo->flinfo->fn_oid) ) );
 		
@@ -369,17 +353,17 @@ Datum taggedtype_out(PG_FUNCTION_ARGS)
 	if( !tti )
 		ereport( ERROR, (errmsg("TaggedType lookup failure: %d", argoid ) ) );
 
-	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
+	tt_class = DetermineTaggedTypeClass( tti );
 	datumSize = datumGetSize( PointerGetDatum( data ), tti->typbyval, tti->typlen );
 
 //	debugDatum( cc(tt_class), data );
 	
 	outstr = DatumGetCString( OidFunctionCall3(tti->typoutput,
-					ExtractTaggedTypeDatum( tt_class, data ),
+					ExtractTaggedTypeDatum( tti, data ),
 					ObjectIdGetDatum(tti->basetype /*typinputoid*/), // Should use getTypeIOParam(...),
 					Int32GetDatum(tti->typmod /*typinputtypmod*/)) );
 
-	tag_text = FindTagByOid( tti->tagtable, ExtractTaggedTypeOid( tt_class, data) );
+	tag_text = FindTagByOid( tti, ExtractTaggedTypeOid( tti, data) );
 	tag = text_to_charp( tag_text );
 	
 	return_str = (char*)tt_palloc( strlen( outstr ) + strlen( tag ) + 1 );
@@ -408,15 +392,15 @@ Datum taggedtype_apply_typmod(PG_FUNCTION_ARGS)
 	elog( NOTICE, "taggedtype_apply_typmod(%p)", DatumGetPointer( datum ) );
 	
 	tti = GetTaggedTypeDetails( get_fn_expr_rettype( fcinfo->flinfo ) );
-	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
+	tt_class = DetermineTaggedTypeClass( tti );
 
-	raw_datum = ExtractTaggedTypeDatum( tt_class, datum );
-	tagoid = ExtractTaggedTypeOid( tt_class, datum ) ;
+	raw_datum = ExtractTaggedTypeDatum( tti, datum );
+	tagoid = ExtractTaggedTypeOid( tti, datum ) ;
 	
 	raw_datum = OidFunctionCall3( tti->typapplytypmod, raw_datum, Int32GetDatum(tti->typmod), BoolGetDatum(explicit) );
 //	debugDatum( tt_class, datum );
 	
-	result_datum = ComposeTaggedType( tt_class, raw_datum, tagoid );
+	result_datum = ComposeTaggedType( tti, raw_datum, tagoid );
 
 //	debugDatum( cc(tt_class), result_datum );
 	
@@ -444,15 +428,15 @@ Datum taggedtype_create(PG_FUNCTION_ARGS)
 	Oid	tagoid;
 	
 	tti = GetTaggedTypeDetails( get_fn_expr_rettype( fcinfo->flinfo ) );
-	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
-	tagoid = FindTagByName( tti->tagtable, tag_text );
+	tt_class = DetermineTaggedTypeClass( tti );
+	tagoid = FindTagByName( tti, tag_text );
 
 //	debugDatum( tt_class, datum );
 	
 	if( tti->typapplytypmod )
 		datum = OidFunctionCall3( tti->typapplytypmod, datum, Int32GetDatum(tti->typmod), BoolGetDatum(true) );
 
-	result_datum = ComposeTaggedType( tt_class, datum, tagoid );
+	result_datum = ComposeTaggedType( tti, datum, tagoid );
 
 //	debugDatum( cc(tt_class), result_datum );
 	
@@ -473,15 +457,15 @@ Datum taggedtype_compose(PG_FUNCTION_ARGS)
 	Oid	tagoid;
 	
 	tti = GetTaggedTypeDetails( get_fn_expr_rettype( fcinfo->flinfo ) );
-	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
-	tagoid = FindTagByName( tti->tagtable, tag_text );
+	tt_class = DetermineTaggedTypeClass( tti );
+	tagoid = FindTagByName( tti, tag_text );
 
 //	debugDatum( tt_class, datum );
 	
 	if( tti->typapplytypmod )
 		datum = OidFunctionCall3( tti->typapplytypmod, datum, Int32GetDatum(tti->typmod), BoolGetDatum(true) );
 
-	result_datum = ComposeTaggedType( tt_class, datum, tagoid );
+	result_datum = ComposeTaggedType( tti, datum, tagoid );
 
 //	debugDatum( cc(tt_class), result_datum );
 	
@@ -507,10 +491,10 @@ Datum taggedtype_extract(PG_FUNCTION_ARGS)
 	
 	// Gets the inputlen and byval of the base type
 	struct TaggedTypeInfo	*tti = GetTaggedTypeDetails( get_fn_expr_argtype( fcinfo->flinfo, 0 ) );
-	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
+	tt_class = DetermineTaggedTypeClass( tti );
 	
-	datums[0] = PointerGetDatum( FindTagByOid( tti->tagtable, ExtractTaggedTypeOid( tt_class, datum ) ) );
-	datums[1] = ExtractTaggedTypeDatum( tt_class, datum );
+	datums[0] = PointerGetDatum( FindTagByOid( tti, ExtractTaggedTypeOid( tti, datum ) ) );
+	datums[1] = ExtractTaggedTypeDatum( tti, datum );
 
 //	debugDatum( tt_class, datums[1] );
 	
@@ -528,9 +512,9 @@ Datum taggedtype_extracttag(PG_FUNCTION_ARGS)
 {
         Datum   datum = PG_GETARG_DATUM(0);
         struct TaggedTypeInfo   *tti = GetTaggedTypeDetails( get_fn_expr_argtype( fcinfo->flinfo, 0 ) );
-	TaggedTypeClass	tt_class = DetermineTaggedTypeClass( tti->typlen, tti->typbyval );
+	TaggedTypeClass	tt_class = DetermineTaggedTypeClass( tti );
 
-	Oid	tagoid = ExtractTaggedTypeOid( tt_class, datum );
+	Oid	tagoid = ExtractTaggedTypeOid( tti, datum );
 	
 	Relation	tt_rel;
 	ScanKeyData	skey;
@@ -605,10 +589,8 @@ Datum create_tagged_type(PG_FUNCTION_ARGS)
 	
 	char	*dll_path, *query;
 	
-	int16	typlen;
-	bool	typbyval;
-	Oid	basetypeoid;
-	int32	basetypetypmod;
+	struct TaggedTypeInfo	tti;
+	
 	Oid	castfuncoid = InvalidOid;
 	
 	TaggedTypeClass	tt_class;
@@ -619,13 +601,13 @@ Datum create_tagged_type(PG_FUNCTION_ARGS)
 	
 	sprintf( newtypename_t, "%s_t", newtypename );
 
-	parseTypeString( basetypename, &basetypeoid, &basetypetypmod );
-//	elog( NOTICE, "Name: %s, Type: %d, Typmod: %d, Table: %d", newtypename, basetypeoid, basetypetypmod, tagtable );
+	parseTypeString( basetypename, &tti.basetype, &tti.typmod );
+//	elog( NOTICE, "Name: %s, Type: %d, Typmod: %d, Table: %d", newtypename, tti.basetype, tti.typmod, tagtable );
 	
-	if( basetypeoid == CSTRINGOID )
+	if( tti.basetype == CSTRINGOID )
 	  ereport( ERROR, (errmsg("Composite CStrings are not allowed")) );
 	  
-	get_typlenbyval( basetypeoid, &typlen, &typbyval );
+	get_typlenbyval( tti.basetype, &tti.typlen, &tti.typbyval );
 /* We don't use SPI anymore, hence we don't care about column name */
 //	if( get_attnum( tagtable, "tag" ) != 1 )
 //	  ereport( ERROR, (errmsg("First column of table '%s' must be named 'tag'", get_rel_name( tagtable ) ) ) );
@@ -635,13 +617,13 @@ Datum create_tagged_type(PG_FUNCTION_ARGS)
 	if( get_attnum( tagtable, "oid" ) == InvalidAttrNumber )
 	  ereport( ERROR, (errmsg("Tag table '%s' must have oids", get_rel_name( tagtable ) ) ) );
 	  
-	tt_class = DetermineTaggedTypeClass( typlen, typbyval );
+	tt_class = DetermineTaggedTypeClass( &tti );
 	
 	tt_SPI_start();
 	  
 	dll_path = GetDLLPath();
 	
-//	elog( NOTICE, "Typlen: %d, Typbyval: %d, DLL path: %s", typlen, typbyval, dll_path );
+//	elog( NOTICE, "Typlen: %d, Typbyval: %d, DLL path: %s", tti.typlen, tti.typbyval, dll_path );
 
 	/********** Type input/output functions and the types themselves ********/
 	query = (char*)palloc( 1024 + strlen( dll_path ) );
@@ -695,17 +677,17 @@ Datum create_tagged_type(PG_FUNCTION_ARGS)
 	elog( NOTICE, "Q: %s", query );
 	SPI_exec( query, 1 );
 
-	if( basetypetypmod != -1 )   /* Base has typmod value, so we need to find the function to apply it */
+	if( tti.typmod != -1 )   /* Base has typmod value, so we need to find the function to apply it */
 	{
 /* Before this catalog the requisite details wern't in pg_cast, and the core
  * hardcoded the functions for precision, so emit a warning */
 
 #if CATALOG_VERSION_NO < 200406160
-		if( basetypeoid == 1042 ) castfuncoid = 668;  /* char() */
-		if( basetypeoid == 1043 ) castfuncoid = 669;  /* varchar() */
-		if( basetypeoid == 1114 ) castfuncoid = 1961; /* timestamp w/o timezone */
-		if( basetypeoid == 1184 ) castfuncoid = 1967; /* timestamp w/ timezone */
-		if( basetypeoid == 1700 ) castfuncoid = 1703; /* numeric */
+		if( tti.basetype == 1042 ) castfuncoid = 668;  /* char() */
+		if( tti.basetype == 1043 ) castfuncoid = 669;  /* varchar() */
+		if( tti.basetype == 1114 ) castfuncoid = 1961; /* timestamp w/o timezone */
+		if( tti.basetype == 1184 ) castfuncoid = 1967; /* timestamp w/ timezone */
+		if( tti.basetype == 1700 ) castfuncoid = 1703; /* numeric */
 		
 		if( !castfuncoid )
 			ereport( WARNING, (errmsg("Cannot enforce precision in result in this version (<7.5)")) );
@@ -713,7 +695,7 @@ Datum create_tagged_type(PG_FUNCTION_ARGS)
 		bool	isNull;
 		
 		sprintf( query, "SELECT castfunc FROM pg_cast WHERE castsource = casttarget and casttarget = %d and castcontext = 'i'",
-				basetypeoid );
+				tti.basetype );
 		elog( NOTICE, "Q: %s", query );
 		if( SPI_exec( query, 1 ) != SPI_OK_SELECT || SPI_processed != 1 )
 			ereport( ERROR, (errmsg("SPI_exec to find castfunc failed") ) );
@@ -728,7 +710,7 @@ Datum create_tagged_type(PG_FUNCTION_ARGS)
 	}
 	/*********** Declare tagged type to backend *********/
 	sprintf( query, "INSERT INTO " TAGGED_TABLE_FULLNAME " (taggedtype, basetype, typmod, tagtable, castfunc) VALUES ('%s','%s',%d,'%s','%d')",
-		    newtypename, basetypename, basetypetypmod, get_rel_name( tagtable ), castfuncoid );
+		    newtypename, basetypename, tti.typmod, get_rel_name( tagtable ), castfuncoid );
 	elog( NOTICE, "Q: %s", query );
 	SPI_exec( query, 1 );
 
@@ -986,29 +968,29 @@ Datum tagged_operator_ttt(PG_FUNCTION_ARGS)
 		ereport( ERROR, (errmsg("tagged_operator_ttt: Not all args tagged") ) );
 	}
 	
-	ttc1 = DetermineTaggedTypeClass( tti1->typlen, tti1->typbyval );
-	ttc2 = DetermineTaggedTypeClass( tti2->typlen, tti2->typbyval );
-	ttcr = DetermineTaggedTypeClass( ttir->typlen, ttir->typbyval );
+	ttc1 = DetermineTaggedTypeClass( tti1 );
+	ttc2 = DetermineTaggedTypeClass( tti2 );
+	ttcr = DetermineTaggedTypeClass( ttir );
 	
-	if( ExtractTaggedTypeOid( ttc1, arg1 ) != ExtractTaggedTypeOid( ttc2, arg2 ) )
+	if( ExtractTaggedTypeOid( tti1, arg1 ) != ExtractTaggedTypeOid( tti2, arg2 ) )
 	{
-		char *tag1 = text_to_charp( FindTagByOid(  tti1->tagtable, ExtractTaggedTypeOid( ttc1, arg1 ) ) );
-		char *tag2 = text_to_charp( FindTagByOid(  tti2->tagtable, ExtractTaggedTypeOid( ttc2, arg2 ) ) );
+		char *tag1 = text_to_charp( FindTagByOid(  tti1, ExtractTaggedTypeOid( tti1, arg1 ) ) );
+		char *tag2 = text_to_charp( FindTagByOid(  tti2, ExtractTaggedTypeOid( tti2, arg2 ) ) );
 		ereport( ERROR, (errmsg( "Using tagged operator %s with incompatable tags (%s,%s)", 
 		           format_operator( tto->tagoper ), tag1, tag2 ) ) );
 	}
 	
-	tag = ExtractTaggedTypeOid( ttc1, arg1 );
+	tag = ExtractTaggedTypeOid( tti1, arg1 );
 	
-	newarg1 = ExtractTaggedTypeDatum( ttc1, arg1 );
-	newarg2 = ExtractTaggedTypeDatum( ttc2, arg2 );
+	newarg1 = ExtractTaggedTypeDatum( tti1, arg1 );
+	newarg2 = ExtractTaggedTypeDatum( tti2, arg2 );
 
 	newres = OidFunctionCall2( tto->basefunc, newarg1, newarg2 );
 
 	if( ttir->typapplytypmod )
 		newres = OidFunctionCall3( ttir->typapplytypmod, newres, Int32GetDatum(ttir->typmod), BoolGetDatum(false) );
 
-	res = ComposeTaggedType( ttcr, newres, tag );
+	res = ComposeTaggedType( ttir, newres, tag );
 
 	PG_RETURN_DATUM( res );
 }
@@ -1042,19 +1024,19 @@ Datum tagged_operator_tot(PG_FUNCTION_ARGS)
 		ereport( ERROR, (errmsg("tagged_operator_tot: Incorrect args tagged") ) );
 	}
 	
-	ttc1 = DetermineTaggedTypeClass( tti1->typlen, tti1->typbyval );
-	ttcr = DetermineTaggedTypeClass( ttir->typlen, ttir->typbyval );
+	ttc1 = DetermineTaggedTypeClass( tti1 );
+	ttcr = DetermineTaggedTypeClass( ttir );
 	
-	tag = ExtractTaggedTypeOid( ttc1, arg1 );
+	tag = ExtractTaggedTypeOid( tti1, arg1 );
 	
-	newarg1 = ExtractTaggedTypeDatum( ttc1, arg1 );
+	newarg1 = ExtractTaggedTypeDatum( tti1, arg1 );
 
 	newres = OidFunctionCall2( tto->basefunc, newarg1, arg2 );
 
 	if( ttir->typapplytypmod )
 		newres = OidFunctionCall3( ttir->typapplytypmod, newres, Int32GetDatum(ttir->typmod), BoolGetDatum(false) );
 
-	res = ComposeTaggedType( ttcr, newres, tag );
+	res = ComposeTaggedType( ttir, newres, tag );
 
 	PG_RETURN_DATUM( res );
 }
@@ -1087,20 +1069,20 @@ Datum tagged_operator_tto(PG_FUNCTION_ARGS)
 		ereport( ERROR, (errmsg("tagged_operator_tto: Incorrect args tagged") ) );
 	}
 	
-	ttc1 = DetermineTaggedTypeClass( tti1->typlen, tti1->typbyval );
-	ttc2 = DetermineTaggedTypeClass( tti2->typlen, tti2->typbyval );
+	ttc1 = DetermineTaggedTypeClass( tti1 );
+	ttc2 = DetermineTaggedTypeClass( tti2 );
 
 	/* Check tags and error on mismatch */	
-	if( ExtractTaggedTypeOid( ttc1, arg1 ) != ExtractTaggedTypeOid( ttc2, arg2 ) )
+	if( ExtractTaggedTypeOid( tti1, arg1 ) != ExtractTaggedTypeOid( tti2, arg2 ) )
 	{
-		char *tag1 = text_to_charp( FindTagByOid(  tti1->tagtable, ExtractTaggedTypeOid( ttc1, arg1 ) ) );
-		char *tag2 = text_to_charp( FindTagByOid(  tti2->tagtable, ExtractTaggedTypeOid( ttc2, arg2 ) ) );
+		char *tag1 = text_to_charp( FindTagByOid(  tti1, ExtractTaggedTypeOid( tti1, arg1 ) ) );
+		char *tag2 = text_to_charp( FindTagByOid(  tti2, ExtractTaggedTypeOid( tti2, arg2 ) ) );
 		ereport( ERROR, (errmsg( "Using tagged operator %s with incompatable tags (%s,%s)", 
 		           format_operator( tto->tagoper ), tag1, tag2 ) ) );
 	}
 	
-	newarg1 = ExtractTaggedTypeDatum( ttc1, arg1 );
-	newarg2 = ExtractTaggedTypeDatum( ttc2, arg2 );
+	newarg1 = ExtractTaggedTypeDatum( tti1, arg1 );
+	newarg2 = ExtractTaggedTypeDatum( tti2, arg2 );
 
 	newres = OidFunctionCall2( tto->basefunc, newarg1, newarg2 );
 
@@ -1136,19 +1118,19 @@ Datum tagged_operator_ott(PG_FUNCTION_ARGS)
 		ereport( ERROR, (errmsg("tagged_operator_ott: Incorrect args tagged") ) );
 	}
 	
-	ttc2 = DetermineTaggedTypeClass( tti2->typlen, tti2->typbyval );
-	ttcr = DetermineTaggedTypeClass( ttir->typlen, ttir->typbyval );
+	ttc2 = DetermineTaggedTypeClass( tti2 );
+	ttcr = DetermineTaggedTypeClass( ttir );
 	
-	tag = ExtractTaggedTypeOid( ttc2, arg2 );
+	tag = ExtractTaggedTypeOid( tti2, arg2 );
 	
-	newarg2 = ExtractTaggedTypeDatum( ttc2, arg2 );
+	newarg2 = ExtractTaggedTypeDatum( tti2, arg2 );
 
 	newres = OidFunctionCall2( tto->basefunc, arg1, newarg2 );
 
 	if( ttir->typapplytypmod )
 		newres = OidFunctionCall3( ttir->typapplytypmod, newres, Int32GetDatum(ttir->typmod), BoolGetDatum(false) );
 
-	res = ComposeTaggedType( ttcr, newres, tag );
+	res = ComposeTaggedType( ttir, newres, tag );
 
 	PG_RETURN_DATUM( res );
 }
